@@ -1,4 +1,19 @@
 # -*- coding: utf-8 -*
+###
+# (C) Copyright [2019] Hewlett Packard Enterprise Development LP
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#   http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+###
 
 """
 connection.py
@@ -14,33 +29,9 @@ from __future__ import unicode_literals
 from builtins import open
 from builtins import str
 from future import standard_library
-from future.utils import raise_from
 
 standard_library.install_aliases()
 
-
-###
-# (C) Copyright (2012-2017) Hewlett Packard Enterprise Development LP
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
-###
-
 import http.client
 import json
 import logging
@@ -49,6 +40,7 @@ import mmap  # so we can upload the iso without having to load it in memory
 import os
 import ssl
 import time
+import traceback
 
 from hpOneView.exceptions import HPOneViewException
 
@@ -56,7 +48,7 @@ logger = logging.getLogger(__name__)
 
 
 class connection(object):
-    def __init__(self, applianceIp, api_version=300):
+    def __init__(self, applianceIp, api_version=800, sslBundle=False, timeout=None):
         self._session = None
         self._host = applianceIp
         self._cred = None
@@ -68,13 +60,15 @@ class connection(object):
         self._proxyHost = None
         self._proxyPort = None
         self._doProxy = False
-        self._sslTrustedBundle = None
         self._sslTrustAll = True
+        self._sslBundle = sslBundle
+        self._sslTrustedBundle = self.set_trusted_ssl_bundle(sslBundle)
         self._nextPage = None
         self._prevPage = None
         self._numTotalRecords = 0
         self._numDisplayedRecords = 0
         self._validateVersion = False
+        self._timeout = timeout
 
     def validateVersion(self):
         version = self.get(uri['version'])
@@ -92,8 +86,9 @@ class connection(object):
         self._doProxy = True
 
     def set_trusted_ssl_bundle(self, sslBundle):
-        self._sslTrustAll = False
-        self._sslTrustedBundle = sslBundle
+        if sslBundle:
+            self._sslTrustAll = False
+        return sslBundle
 
     def get_session(self):
         return self._session
@@ -148,6 +143,9 @@ class connection(object):
                     conn.close()
                 time.sleep(1)
                 continue
+            except http.client.HTTPException:
+                raise HPOneViewException('Failure during login attempt.\n %s' % traceback.format_exc())
+
         return resp, body
 
     def download_to_stream(self, stream_writer, url, body='', method='GET', custom_headers=None):
@@ -168,6 +166,13 @@ class connection(object):
                 if resp.status >= 400:
                     self.__handle_download_error(resp, conn)
 
+                if resp.status == 302:
+                    return self.download_to_stream(stream_writer=stream_writer,
+                                                   url=resp.getheader('Location'),
+                                                   body=body,
+                                                   method=method,
+                                                   custom_headers=http_headers)
+
                 tempbytes = True
                 while tempbytes:
                     tempbytes = resp.read(chunk_size)
@@ -182,6 +187,8 @@ class connection(object):
                     conn.close()
                 time.sleep(1)
                 continue
+            except http.client.HTTPException:
+                raise HPOneViewException('Failure during login attempt.\n %s' % traceback.format_exc())
 
         return successful_connected
 
@@ -209,22 +216,27 @@ class connection(object):
             context.load_verify_locations(self._sslTrustedBundle)
             if self._doProxy is False:
                 conn = http.client.HTTPSConnection(self._host,
-                                                   context=context)
+                                                   context=context,
+                                                   timeout=self._timeout)
             else:
                 conn = http.client.HTTPSConnection(self._proxyHost,
                                                    self._proxyPort,
-                                                   context=context)
+                                                   context=context,
+                                                   timeout=self._timeout)
                 conn.set_tunnel(self._host, 443)
         else:
             context.verify_mode = ssl.CERT_NONE
             if self._doProxy is False:
                 conn = http.client.HTTPSConnection(self._host,
-                                                   context=context)
+                                                   context=context,
+                                                   timeout=self._timeout)
             else:
                 conn = http.client.HTTPSConnection(self._proxyHost,
                                                    self._proxyPort,
-                                                   context=context)
+                                                   context=context,
+                                                   timeout=self._timeout)
                 conn.set_tunnel(self._host, 443)
+
         return conn
 
     def _open(self, name, mode):
@@ -247,8 +259,7 @@ class connection(object):
         fout = self._open(files + '.b64', 'wb')
         fout.write(bytearray('--' + BOUNDARY + CRLF, 'utf-8'))
         fout.write(bytearray('Content-Disposition: form-data'
-                             '; name="file"; filename="' +
-                             baseName + '"' + CRLF, "utf-8"))
+                             '; name="file"; filename="' + baseName + '"' + CRLF, "utf-8"))
         fout.write(bytearray('Content-Type: application/octet-stream' + CRLF,
                              'utf-8'))
         fout.write(bytearray(CRLF, 'utf-8'))
@@ -368,55 +379,6 @@ class connection(object):
     def patch(self, uri, body, custom_headers=None):
         return self.__do_rest_call('PATCH', uri, body, custom_headers=custom_headers)
 
-    def get_entities_byrange(self, uri, field, xmin, xmax, count=-1):
-        new_uri = uri + '?filter="\'' + field + '\'%20>%20\'' + xmin \
-                      + '\'"&filter="\'' + field + '\'%20<%20\'' + xmax \
-                      + '\'"&start=0&count=' + str(count)
-        body = self.get(new_uri)
-        return get_members(body)
-
-    def get_entities_byfield(self, uri, field, value, count=-1):
-        new_uri = uri + '?start=0&count=' + str(count) \
-                      + '&filter=' + field + '=\'' + value + '\''
-        try:
-            body = self.get(new_uri)
-        except:
-            print(new_uri)
-            raise
-        return get_members(body)
-
-    def get_entity_byfield(self, uri, field, value, count=-1):
-        new_uri = uri + '?filter="\'' + field + '\'%20=%20\'' + value \
-                      + '\'"&start=0&count=' + str(count)
-
-        try:
-            body = self.get(new_uri)
-        except:
-            print(new_uri)
-            raise
-        return get_member(body)
-
-    def conditional_post(self, uri, body):
-        try:
-            task, entity = self.post(uri, body)
-        except HPOneViewException as e:
-            # this may have failed because the entity already exists,
-            # unfortunately there is not a uniform way to report this,
-            # so we just try to find an existing entity with the same name
-            # and return it assuming all names are unique (which is a
-            # reasonable assumption)
-            if 'DUPLICATE' in e.errorCode and 'NAME' in e.errorCode:
-                try:
-                    entity = self.get_entity_byfield(uri, 'name', body['name'])
-                except Exception:
-                    # Didn't find the entity, raise exception
-                    raise e
-                if not entity:
-                    raise e
-            else:
-                raise e
-        return entity
-
     def __body_content_is_task(self, body):
         return isinstance(body, dict) and 'category' in body and body['category'] == 'tasks'
 
@@ -483,12 +445,17 @@ class connection(object):
         try:
             if self._validateVersion is False:
                 self.validateVersion()
-        except Exception as e:
-            raise_from(HPOneViewException('Failure during login attempt.'), e)
+        except Exception:
+            raise(HPOneViewException('Failure during login attempt.\n %s' % traceback.format_exc()))
 
         self._cred = cred
         try:
-            task, body = self.post(uri['loginSessions'], self._cred)
+            if self._cred.get("sessionID"):
+                self.set_session_id(self._cred["sessionID"])
+                task, body = self.put(uri['loginSessions'], None)
+            else:
+                self._cred.pop("sessionID", None)
+                task, body = self.post(uri['loginSessions'], self._cred)
         except HPOneViewException:
             logger.exception('Login failed')
             raise
@@ -673,7 +640,11 @@ uri = {
     # ------------------------------------
     # Uncategorized
     # ------------------------------------
-    'unmanaged-devices': '/rest/unmanaged-devices'
+    'unmanaged-devices': '/rest/unmanaged-devices',
+    # ------------------------------------
+    # Hypervisors
+    # ------------------------------------
+    'hypervisor-managers': '/rest/hypervisor-managers'
 }
 
 ############################################################################
